require 'spec_helper'

describe BranchableCDNAssets::CheckBefore do

  class Stub
    extend BranchableCDNAssets::CheckBefore

    def hello; 'world'; end
    def filter;   true; end
  end

  after :each do
    Stub.remove_instance_variable :@_before_checks
  end

  describe "::before_checks" do
    it "returns an array if @_before_checks not set" do
      expect( Stub.before_checks.is_a?(Array) ).to be_truthy
    end

    it "returs @_before_checks if set" do
      Stub.instance_variable_set :@_before_checks, ['hello']
      expect( Stub.before_checks ).to match_array ['hello']
    end
  end

  describe "::check_before" do
    it "adds hash with params to before_checks" do
      Stub.check_before :check, {hello: 'world'}
      expect( Stub.before_checks.first ).to eq check: :check, hello: 'world'
    end
  end

  describe "#with_check" do

    it "sends the the called method" do
      stub = Stub.new
      expect( stub ).to receive(:hello)
      stub.with_check( :hello )
    end

    context "with check defined on method" do
      before :each do
        Stub.instance_eval do; check_before :filter, methods: [:hello]; end
      end

      it "runs included checks" do
        stub = Stub.new
        expect( stub ).to receive(:filter)
        stub.with_check(:hello)
      end

      it "runs the called method" do
        stub = Stub.new
        expect( stub ).to receive(:hello)
        stub.with_check(:hello)
      end
    end
  end

end