require 'spec_helper'

describe BranchableCDNAssets::FileManager do

  describe "#find" do

    before :each do
      Given.fixture 'base'
      Given.file 'cdn/master.manifest', "image_one\nimage_remote_one\n"
      Given.file 'cdn/staging.manifest', "image_one\nimage_remote_two\n"
      Given.file 'cdn/image_one', ''
      Given.file 'cdn/image_two', ''
    end

    context "when on production branch" do
      before :each do
        allow( Asgit ).to receive(:current_branch)
                      .and_return('master')
        @manager = described_class.new BranchableCDNAssets::Config.new('config/cdn_assets.yaml')
      end

      it "returns :local if asset is local" do
        expect( @manager.find('image_one') ).to eq :local
      end

      it "returns the production url of the asset" do
        expect( @manager.find('image_remote_one') ).to eq 'http://production.com/image_remote_one'
      end

      it "returns nil if asset is missing" do
        expect( @manager.find('missing') ).to be_nil
      end

      it "returns :local if told to look local and file exists" do
        expect( @manager.find('image_one', :local) ).to eq :local
      end

      it "returns nil if told to look local and file only on remote" do
        expect( @manager.find('image_remote_one', :local) ).to be_nil
      end

      it "returns remote file if told to look remote and file exists on both" do
        expect( @manager.find('image_one', :remote) ).to eq 'http://production.com/image_one'
      end

      it "returns nil if told to look remote and file only local" do
        expect( @manager.find('image_two', :remote) ).to be_nil
      end
    end

    context "when on staging branch" do
      before :each do
        allow( Asgit ).to receive(:current_branch)
                      .and_return('staging')
        @manager = described_class.new BranchableCDNAssets::Config.new('config/cdn_assets.yaml')
      end

      it "returns :local if asset is local" do
        expect( @manager.find('image_one') ).to eq :local
      end

      it "returns namespaced staging url if listed in staging manifest" do
        expect( @manager.find('image_remote_two') ).to eq 'http://staging.com/staging/image_remote_two'
      end

      it "returns production url if listed only in production manifest" do
        expect( @manager.find('image_remote_one') ).to eq 'http://production.com/image_remote_one'
      end

      it "returns nil if asset is missing" do
        expect( @manager.find('missing') ).to be_nil
      end

      it "returns :local if told to look local and file exists" do
        expect( @manager.find('image_one', :local) ).to eq :local
      end

      it "returns nil if told to look local and file only on remote" do
        expect( @manager.find('image_remote_one', :local) ).to be_nil
      end

      it "returns remote file if told to look remote and file exists on both" do
        expect( @manager.find('image_one', :remote) ).to eq 'http://staging.com/staging/image_one'
      end

      it "returns production file if told to look remote and file exists only on prod" do
        expect( @manager.find('image_remote_one', :remote) ).to eq 'http://production.com/image_remote_one'
      end

      it "returns nil if told to look remote and file only local" do
        expect( @manager.find('image_two', :remote) ).to be_nil
      end
    end

  end

end
