require 'spec_helper'

describe BranchableCDNAssets::Invalidator::Base do

  after :each do
    Object.send(:remove_const, :Foo) if defined? Foo
  end

  describe "::register_as" do
    it "calls register with given key on BranchableCDNAssets::Invalidator" do
      class Foo < BranchableCDNAssets::Invalidator::Base; end
      expect( BranchableCDNAssets::Invalidator ).to receive(:register)
                                                .with(:foo, Foo)

      class Foo
        register_as :foo
      end
    end
  end

  describe "::require_keys" do
    it "sets class.required_keys" do
      class Foo < BranchableCDNAssets::Invalidator::Base
        require_keys [:foo, :bar]
      end

      expect( Foo.required_keys ).to match_array [:foo, :bar]
    end
  end

  describe "::set_defaults" do
    it "sets class.defaults" do
      class Foo < BranchableCDNAssets::Invalidator::Base
        set_defaults foo: 'bar'
      end

      expect( Foo.defaults ).to eq foo: 'bar'
    end
  end

  describe "#initialize" do
    it "takes a hash and converts to config struct" do
      config = {
        foo: 'bar',
        wu:  'tang'
      }

      subject = described_class.new(config)

      expect( subject.config.foo ).to eq 'bar'
      expect( subject.config.wu ).to  eq 'tang'
    end

    it "uses defaults for key if key not given" do
      class Foo < BranchableCDNAssets::Invalidator::Base
        set_defaults foo: 'bar'
      end

      subject = Foo.new baz: 'bang'
      expect( subject.config.foo ).to eq 'bar'
    end

    it "given value even if default given" do
      class Foo < BranchableCDNAssets::Invalidator::Base
        set_defaults foo: 'bar'
      end

      subject = Foo.new foo: 'bang'
      expect( subject.config.foo ).to eq 'bang'
    end

    it "raises ArgumentError if missing required keys" do
      class Foo < BranchableCDNAssets::Invalidator::Base
        require_keys [:boom]
      end

      expect{
        Foo.new foo: 'bar'
      }.to raise_error ArgumentError
    end
  end

  describe "#invalidate_files" do
    it "raises a NoMethodError, it's not implemented" do
      expect{
        described_class.new.invalidate_files('foo')
      }.to raise_error NoMethodError
    end
  end

end
