require 'spec_helper'

describe BranchableCDNAssets::Invalidator::Cloudfront do

  let(:config) do
    {
      distribution_id: 'dist_id',
      access_key: 'access_key',
      secret_key: 'secret_key'
    }
  end

  describe "#initialize" do

    before :each do
      @subject = described_class.new(config)
    end

    it "sets #distribution_id" do
      expect( @subject.config.distribution_id ).to eq 'dist_id'
    end
    it "sets #access_key" do
      expect( @subject.config.access_key ).to eq 'access_key'
    end
    it "sets #secret_key" do
      expect( @subject.config.secret_key ).to eq 'secret_key'
    end

  end

  describe "#cdn" do
    it "sets #cdn with a Fog instance" do
      fog = double("Fog::CDN")
      expect( Fog::CDN ).to receive(:new).with(
        provider: 'AWS',
        aws_access_key_id: 'access_key',
        aws_secret_access_key: 'secret_key'
      ).and_return(fog)
      expect( described_class.new(config).cdn ).to eq fog
    end
  end

  describe "#invalidate_files" do
    before :each do
      @fog = double("Fog::CDN")
      @resp = double( "Response",
                       body: { "InvalidationBatch" => {
                         "Path" => ['file_one','file_two']
                       }
                    })
      allow( Fog::CDN ).to receive(:new).with(
        provider: 'AWS',
        aws_access_key_id: 'access_key',
        aws_secret_access_key: 'secret_key'
      ).and_return(@fog)
      @subject = described_class.new(config)
    end

    it "calls cloudfront to invalidate given files" do
      expect( @fog ).to receive(:post_invalidation)
                    .with( 'dist_id', ['file_one','file_two'])
                    .and_return(@resp)

      @subject.invalidate_files ['file_one','file_two']
    end
  end


end