require 'spec_helper'

describe BranchableCDNAssets::Invalidator do

  def clean_invalidators
    described_class.instance_variable_set :@_invalidators, nil
  end

  def clean_foo_class
    Object.send(:remove_const, :Foo) if defined? Foo
  end

  before :each do
    clean_invalidators
  end

  after :each do
    clean_foo_class
  end

  describe "::invalidators" do
    it "returns empty hash if no invalidators are registered" do
      expect( described_class.invalidators ).to eq({})
    end
    it "returns hash of registered invalidators" do
      class Foo < BranchableCDNAssets::Invalidator::Base
        register_as :foo
      end
      expect( described_class.invalidators ).to eq({foo: Foo})
    end
  end

  describe "::register" do
    it "adds invalidator to invalidators hash" do
      class Foo; end
      described_class.register :foo, Foo
      expect( described_class.invalidators ).to eq({foo: Foo})
    end
  end

  describe "::find" do
    it "returns invalidator class if one is registered" do
      class Foo; end
      described_class.register :foo, Foo

      expect( described_class.find(:foo) ).to eq Foo
    end
    it "returns false if invalidator isn't registered" do
      expect( described_class.find(:bar) ).to eq false
    end
  end

end