require 'spec_helper'
require 'branchable_cdn_assets/rake_tasks'

describe BranchableCDNAssets::RakeTasks do

  describe "::register" do

    let(:data) do
      [
        environments: {
          production: {
            host: 'production',
            url:  'http://production.com',
            root: '/var/www/production'
          },
          default: {
            host: 'default',
            url:  'http://default.com',
            root: '/var/www/default'
          }
        }
      ]
    end

    before :each do
      Given.fixture 'base'
      Rake::Task.clear
    end

    after :each do
      Given.cleanup!
    end

    it "adds tasks under provided namespace" do
      described_class.register(:test, *data)

      expect( Rake::Task.tasks.map(&:to_s) ).to include 'test:list'
    end

    it "sends method with check to the file_manager" do
      class FileManagerStub
        extend BranchableCDNAssets::CheckBefore

        [:list, :pull!].each do |m|
          define_method m do |*args|
            nil
          end
        end
      end

      file_mananger_double = FileManagerStub.new
      allow( BranchableCDNAssets::FileManager ).to receive(:new).with(anything()).and_return( file_mananger_double )

      expect( file_mananger_double ).to receive(:list)
      expect( file_mananger_double ).to receive(:pull!)

      described_class.register(:cdn, *data)
      Rake::Task['cdn:list'].invoke
      Rake::Task['cdn:pull'].invoke
    end

  end

end